package com.perforce.client.api;

class P4Error
{
    //================================//
    //
    // PACKAGE
    //
    //================================//
    
    long instance = 0; // the native perr instance of the error
    
    int id()
    {
        return nId(this.instance);
    }

    String fmt()
    {
        return nFmt(this.instance);
    }
    
    /**
     * Test to see if there was an error made.
     *
     * @return if there is an error or not.
     *
     **/
    boolean test()
    {
        return nTest(this.instance);
    }

    P4Error()
    {
        this.instance = nNewInstance();
        javaCreatedTheInstance = true;
    }
    
    /**
     * Make a new P4Error that will just bind to an already
     * existing native Error.
     *
     * @param instance a native peer to bind this to.
     * @return the java interface.
     *
     **/
    static P4Error makeJavaPeer(long instance)
    {
        return new P4Error(instance);
    }
    
    
    /**
     * Take a P4ClientException, and make sure that the
     * error on the native side is set.
     *
     * <p>You would call this when you've caught a P4ClientException
     * and want to convey that to the native side</p>
     *
     * @param err the exception that we use to see what to set.
     *
     **/
    void convertException(P4ClientException err)
    {
        //
        // We can get a unique name for this exception
        //
        String kind = err.getClass().getName();
        
        //
        // If we got one (and we always should)
        //
        if (kind != null)
        {
            //
            // Get the native ErrorId and set it on the native side
            //
            int id = err.getErrorId();
            nSetErrorId(this.instance, id);
        }
        else
        {
            //
            // By default, just set a message
            //
            nSetMessage(this.instance, err.toString());
        }
    }
    

    /**
     * Given a P4Error, see if there was an exception on the native side.
     * If so, throw the corresponding P4ClientException.
     *
     * <p>You would use this if you wanted to convert from P4's Error
     * handling system to a more java-like exception mechanism.</p>
     *
     * @throws the corresponding P4ClientException that matches the Error
     *         but only if one needs to be thrown.
     *
     **/
    void checkException() throws P4ClientException
    {
        //
        // Check to see if there was an error.
        //
        if (test())
        {
            //
            // There was, so get the native one and, as best we
            // can, throw the corresponding exception.
            //
            P4ClientException exp = new P4ClientException(id(), fmt());
            
            if (exp != null)
            {
                throw exp;
            }
        }
    }

     

    //================================//
    //
    // PROTECTED
    //
    //================================//

    protected void finalize()
    {
        //
        // If this instance was create to pass to the native side,
        // then a native pointer was allocated and we need to clean
        // that up, otherwise, this was created as a face to an
        // Error that was created outside of Java and we do not want
        // to clean that up.
        //
        if (javaCreatedTheInstance)
        {
            nDeleteInstance(this.instance);
        }
    }

    
    //================================//
    //
    // PRIVATE
    //
    //================================//

    // to track if we created this or the native side did
    //
    private boolean javaCreatedTheInstance;
        
    //
    // methods to create and dispose of native Error pointers.
    //
    private static native long nNewInstance();
    private static native void nDeleteInstance(long instance);
      

    /**
     * Make a P4Error to use as a "face" to an actual, pre-existing
     * native Error instance.
     *
     * @param instance the pre-existing native instance.
     *
     **/
    private P4Error(long instance)
    {
        this.instance = instance;
        javaCreatedTheInstance = false;
    }


    //================================//
    //
    // NATIVE
    //
    //================================//
   
    private static native void nSetErrorId(long errInstance, int errorID);
    private static native void nSetMessage(long errInstance, String message);
    
    private static native boolean nTest(long instance);
    private static native String nFmt(long instance);
    private static native int nId(long instance);
}
